# $Id: Charset.pm,v 1.2 1998/10/24 18:13:10 arlexa Exp $
#
# Apache::Charset.xs - mod_perl interface to Russian Apache (http://apache.lexa.ru)
# internals
#
# Copyright (C) 1998, Alex Tutubalin <lexa@lexa.ru>
#




package Apache::Charset;

use strict;
use vars qw($VERSION @ISA);
use Apache;
use DynaLoader ();

@ISA = qw(DynaLoader);
$VERSION = '0.11';


=head1 NAME

Apache::Charset - mod_perl interface to Russian Apache internals

=head1 SYNOPSIS

    use Apache::Charset;
    print "Known charsets: ".join(" ",$r->charsets())."<br>\n";
    print "Server charset: ".$r->server_charset()."<br>\n";
    print "Client charset: ".$r->client_charset()."<br>\n";
   
    $str="some string";
    $str_1251 = $r->recode($r->server_charset(),"windows-1251",$str);
    $esc="%AA%BB%CC%DD%EE%FF";
    $esc_1251=$r->recode_esc($r->server_charset(),"windows-1251",$esc);

=head1 DESCRIPTION

Apache::Charset provides interface to Russian Apache functions.
This module  should be called after fixup stage of apache handlers
(i.e. one should use Apache::Charset in Apache::Registry scripts or in
PerlHandler routines).

=cut

  if ($ENV{MOD_PERL}) {
    bootstrap Apache::Charset $VERSION;
  } else {
    #die "You should run Apache::Charset under mod_perl";
  }



sub new 
  {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless $self,$class;
    $self->{request} = Apache->request;
    $self;
  }




sub set_request 
  {
    my ($self,$req) = @_;
    warn "You should call set_request(some_object_of_Apache_type)" unless ref($req) eq "Apache";
    $self->{request} = $req;
  }



sub is_ok 
  {
    my $self = shift;
    return xs_is_ok($self->{request});
  }



sub server_charset 
  {
    my $self = shift;
    return $self->is_ok?xs_server_charset($self->{request}):undef;
  }




sub client_charset 
  {
    my $self = shift;
    return $self->is_ok?xs_client_charset($self->{request}):undef
  }



sub charsets
  {
    my $self = shift;
    my @charsets = ();
    return undef unless $self->is_ok;

    my $max = xs_no_of_charsets($self->{request});
    for (my $i=0; $i < $max ; $i++) {
      my $name = xs_nth_charset_name($self->{request},$i);
      last unless $name;
      push @charsets,$name;
    }
    return @charsets;
  }


sub server2client
  {
    my ($self,$sc) = @_;
    return ($sc && $self->is_ok)?xs_server2client($self->{request},$sc):undef;
  }

sub server2client_esc
  {
    my ($self,$sc) = @_;
    return ($sc && $self->is_ok)? xs_server2client_esc($self->{request},$sc):undef;
  }

sub client2server
  {
    my ($self,$sc) = @_;
    return ($sc && $self->is_ok)? xs_client2server($self->{request},$sc): undef;
  }

sub client2server_esc
  {
    my ($self,$sc) = @_;
    return ($sc && $self->is_ok)? xs_client2server_esc($self->{request},$sc):undef;
  }

sub recode 
  {
    my ($self,$from,$to,$str)=@_;
    return undef unless $self->is_ok;
    my $p = xs_create_subpool($self->{request});
    my $ret = xs_recode_str($self->{request},$p,$from,$to,$str);
    xs_delete_subpool($p);
    return $ret;
  }

sub recode_esc
  {
    my ($self,$from,$to,$str)=@_;
    return undef unless $self->is_ok;

    my $p = xs_create_subpool($self->{request});
    my $ret = xs_recode_str_esc($self->{request},$p,$from,$to,$str);
    xs_delete_subpool($p);
    return $ret;
  }



1;

__END__

=head1 MEMBER FUNCTIONS

=over

=item B<$recoder = new Apache::Charset();>

Creates new B<Apache::Charset> object with internal request_rec record  set to current request.

=item B<$recoder-E<gt>set_request($r);>

Sets internal request_rec reference to B<$r>. B<$r> should be an object returned by
Apache->request call.

=item B<$recoder-E<gt>is_ok;>

Returns 1 if Russian Apache module has successfully processed current request and
undef overwise (f.e. mod_charset is disabled, or some recoding tables was not found etc)

=item B<$server_charset=$recoder-E<gt>server_charset;>

Returns canonical name of server (or "source") charset. Returned value is equivalent to 
B<$ENV{SOURCE_CHARSET}>. Will return B<undef> if B<$recoder> is not ok.

=item B<$client_charset=$recoder-E<gt>client_charset;>

Returns canonical name of client  charset. Returned value is equivalent to B<$ENV{CHARSET}>.
Will return B<undef> if B<$recoder> is not ok.

=item B<@charsets=$recoder-E<gt>charsets();>

Returns list of charsets known by server (i.e. configured by B<CharsetRecodeTable> directives).

=item B<$str=$recoder-E<gt>server2client($old_string);> 

=item B<$str=$recoder-E<gt>server2client_esc($old_string);>

Returns string converted from server charset to client charset. server2_client_esc recodes %AA-escapes.

=item B<$str=$recoder-E<gt>client2server($old_string);> 

=item B<$str=$recoder-E<gt>client2server_esc($old_string);>

Returns string converted from client charset to server charset. server2_client_esc recodes %AA-escapes.

=item B<$str=$recoder-E<gt>recode($from,$to,$string);>

=item B<$str=$recoder-E<gt>recode_esc($from,$to,$string);>

Generic recoder interface. Recodes $string from $from charset into $to charset

=head1 BUGS


=head1 SEE ALSO

L<mod_perl(3)>, L<Apache(3)>


=head1 AUTHOR

Alex Tutubalin E<lt>lexa@lexa.ruE<gt>


=cut
